@@ -1,65 +1,52 @@
 /*
  * The XML and API file generator module for SIP.
  *
- * Copyright (c) 2015 Riverbank Computing Limited <info@riverbankcomputing.com>
- *
+ * Copyright (c) 2007
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
+ * 
  * This file is part of SIP.
- *
+ * 
  * This copy of SIP is licensed for use under the terms of the SIP License
  * Agreement.  See the file LICENSE for more details.
- *
- * This copy of SIP may also used under the terms of the GNU General Public
- * License v2 or v3 as published by the Free Software Foundation which can be
- * found in the files LICENSE-GPL2 and LICENSE-GPL3 included in this package.
- *
+ * 
  * SIP is supplied WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 
 #include <stdio.h>
-#include <string.h>
 
 #include "sip.h"
 
 
 #define XML_VERSION_NR  0       /* The schema version number. */
 
-/* Icon numbers.  The values are those used by the eric IDE. */
-#define CLASS_ID        1
-#define METHOD_ID       4
-#define VARIABLE_ID     7
-#define ENUM_ID         10
-
 
 static void apiEnums(sipSpec *pt, moduleDef *mod, classDef *scope, FILE *fp);
 static void apiVars(sipSpec *pt, moduleDef *mod, classDef *scope, FILE *fp);
-static int apiCtor(sipSpec *pt, moduleDef *mod, classDef *scope, ctorDef *ct,
-        int sec, FILE *fp);
-static int apiOverload(sipSpec *pt, moduleDef *mod, classDef *scope,
-        overDef *od, int sec, FILE *fp);
-static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
-        int sec, int names, int defaults, int in_str, FILE *fp);
+static int apiCtor(moduleDef *mod, classDef *scope, ctorDef *ct, int sec,
+        FILE *fp);
+static int apiOverload(moduleDef *mod, classDef *scope, overDef *od, int sec,
+        FILE *fp);
+static int apiArgument(argDef *ad, int out, int need_comma, int sec, FILE *fp);
 static void xmlClass(sipSpec *pt, moduleDef *mod, classDef *cd, FILE *fp);
 static void xmlEnums(sipSpec *pt, moduleDef *mod, classDef *scope, int indent,
         FILE *fp);
 static void xmlVars(sipSpec *pt, moduleDef *mod, classDef *scope, int indent,
         FILE *fp);
-static void xmlFunction(sipSpec *pt, classDef *scope, memberDef *md,
-        overDef *oloads, int indent, FILE *fp);
-static int xmlCtor(sipSpec *pt, classDef *scope, ctorDef *ct, int sec,
+static void xmlFunction(classDef *scope, memberDef *md, overDef *oloads,
         int indent, FILE *fp);
-static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
-        overDef *od, classDef *xtnds, int stat, int sec, int indent, FILE *fp);
+static int xmlCtor(classDef *scope, ctorDef *ct, int sec, int indent, FILE *fp);
+static int xmlOverload(classDef *scope, memberDef *md, overDef *od,
+        classDef *xtnds, int stat, int sec, int indent, FILE *fp);
 static void xmlCppSignature(FILE *fp, overDef *od);
-static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
-        int sec, int indent, FILE *fp);
-static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp);
+static void xmlArgument(argDef *ad, const char *dir, int res_xfer, int sec,
+        int indent, FILE *fp);
+static void xmlType(argDef *ad, int sec, FILE *fp);
 static void xmlIndent(int indent, FILE *fp);
 static const char *dirAttribute(argDef *ad);
-static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope);
-static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
-        int sec, int names, int defaults, int in_str, int is_signal);
+static void exportDefaultValue(argDef *ad, FILE *fp);
+static const char *pyType(argDef *ad, int sec, classDef **scope);
 
 
 /*
@@ -86,8 +73,8 @@ void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile)
         if (od->common->slot != no_slot)
             continue;
 
-        if (apiOverload(pt, mod, NULL, od, FALSE, fp))
-            apiOverload(pt, mod, NULL, od, TRUE, fp);
+        if (apiOverload(mod, NULL, od, FALSE, fp))
+            apiOverload(mod, NULL, od, TRUE, fp);
     }
 
     for (cd = pt->classes; cd != NULL; cd = cd->next)
@@ -108,8 +95,8 @@ void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile)
             if (isPrivateCtor(ct))
                 continue;
 
-            if (apiCtor(pt, mod, cd, ct, FALSE, fp))
-                apiCtor(pt, mod, cd, ct, TRUE, fp);
+            if (apiCtor(mod, cd, ct, FALSE, fp))
+                apiCtor(mod, cd, ct, TRUE, fp);
         }
 
         for (od = cd->overs; od != NULL; od = od->next)
@@ -120,8 +107,8 @@ void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile)
             if (od->common->slot != no_slot)
                 continue;
 
-            if (apiOverload(pt, mod, cd, od, FALSE, fp))
-                apiOverload(pt, mod, cd, od, TRUE, fp);
+            if (apiOverload(mod, cd, od, FALSE, fp))
+                apiOverload(mod, cd, od, TRUE, fp);
         }
     }
 
@@ -132,15 +119,15 @@ void generateAPI(sipSpec *pt, moduleDef *mod, const char *apiFile)
 /*
  * Generate an API ctor.
  */
-static int apiCtor(sipSpec *pt, moduleDef *mod, classDef *scope, ctorDef *ct,
-        int sec, FILE *fp)
+static int apiCtor(moduleDef *mod, classDef *scope, ctorDef *ct, int sec,
+        FILE *fp)
 {
     int need_sec = FALSE, need_comma, a;
 
     /* Do the callable type form. */
     fprintf(fp, "%s.", mod->name);
-    prScopedPythonName(fp, scope->ecd, scope->pyname->text);
-    fprintf(fp, "?%d(", CLASS_ID);
+    prScopedPythonName(fp, scope->ecd, scope->pyname);
+    fprintf(fp, "(");
 
     need_comma = FALSE;
 
@@ -148,8 +135,7 @@ static int apiCtor(sipSpec *pt, moduleDef *mod, classDef *scope, ctorDef *ct,
     {
         argDef *ad = &ct->pysig.args[a];
 
-        need_comma = apiArgument(pt, ad, FALSE, need_comma, sec, TRUE, TRUE,
-                FALSE, fp);
+        need_comma = apiArgument(ad, FALSE, need_comma, sec, fp);
 
         if (ad->atype == rxcon_type || ad->atype == rxdis_type)
             need_sec = TRUE;
@@ -159,12 +145,11 @@ static int apiCtor(sipSpec *pt, moduleDef *mod, classDef *scope, ctorDef *ct,
 
     /* Do the call __init__ form. */
     fprintf(fp, "%s.", mod->name);
-    prScopedPythonName(fp, scope->ecd, scope->pyname->text);
-    fprintf(fp, ".__init__?%d(self", CLASS_ID);
+    prScopedPythonName(fp, scope->ecd, scope->pyname);
+    fprintf(fp, ".__init__(self");
 
     for (a = 0; a < ct->pysig.nrArgs; ++a)
-        apiArgument(pt, &ct->pysig.args[a], FALSE, TRUE, sec, TRUE, TRUE,
-                FALSE, fp);
+        apiArgument(&ct->pysig.args[a], FALSE, TRUE, sec, fp);
 
     fprintf(fp, ")\n");
 
@@ -193,14 +178,14 @@ static void apiEnums(sipSpec *pt, moduleDef *mod, classDef *scope, FILE *fp)
         {
             fprintf(fp, "%s.", mod->name);
             prScopedPythonName(fp, ed->ecd, ed->pyname->text);
-            fprintf(fp, "?%d\n", ENUM_ID);
+            fprintf(fp, "\n");
         }
 
         for (emd = ed->members; emd != NULL; emd = emd->next)
         {
             fprintf(fp, "%s.", mod->name);
             prScopedPythonName(fp, ed->ecd, emd->pyname->text);
-            fprintf(fp, "?%d\n", ENUM_ID);
+            fprintf(fp, "\n");
         }
     }
 }
@@ -223,7 +208,7 @@ static void apiVars(sipSpec *pt, moduleDef *mod, classDef *scope, FILE *fp)
 
         fprintf(fp, "%s.", mod->name);
         prScopedPythonName(fp, vd->ecd, vd->pyname->text);
-        fprintf(fp, "?%d\n", VARIABLE_ID);
+        fprintf(fp, "\n");
     }
 }
 
@@ -231,17 +216,68 @@ static void apiVars(sipSpec *pt, moduleDef *mod, classDef *scope, FILE *fp)
 /*
  * Generate a single API overload.
  */
-static int apiOverload(sipSpec *pt, moduleDef *mod, classDef *scope,
-        overDef *od, int sec, FILE *fp)
+static int apiOverload(moduleDef *mod, classDef *scope, overDef *od, int sec,
+        FILE *fp)
 {
-    int need_sec;
+    int need_sec = FALSE, need_comma = FALSE, is_res, nr_out, a;
 
     fprintf(fp, "%s.", mod->name);
     prScopedPythonName(fp, scope, od->common->pyname->text);
-    fprintf(fp, "?%d", METHOD_ID);
+    fprintf(fp, "(");
 
-    need_sec = exportPythonSignature(pt, fp, &od->pysig, sec, TRUE, TRUE,
-            FALSE, FALSE);
+    if (scope != NULL && scope->iff->type != namespace_iface && !isStatic(od))
+    {
+        fprintf(fp, "self");
+        need_comma = TRUE;
+    }
+
+    nr_out = 0;
+
+    for (a = 0; a < od->pysig.nrArgs; ++a)
+    {
+        argDef *ad = &od->pysig.args[a];
+
+        if (isOutArg(ad))
+            ++nr_out;
+
+        if (!isInArg(ad))
+            continue;
+
+        need_comma = apiArgument(ad, FALSE, need_comma, sec, fp);
+
+        if (ad->atype == rxcon_type || ad->atype == rxdis_type)
+            need_sec = TRUE;
+    }
+
+    fprintf(fp, ")");
+
+    is_res = (od->pysig.result.atype != void_type || od->pysig.result.nrderefs != 0);
+
+    if (is_res || nr_out > 0)
+    {
+        fprintf(fp, " -> ");
+
+        if ((is_res && nr_out > 0) || nr_out > 1)
+            fprintf(fp, "(");
+
+        if (is_res)
+            need_comma = apiArgument(&od->pysig.result, TRUE, FALSE, sec, fp);
+        else
+            need_comma = FALSE;
+
+        for (a = 0; a < od->pysig.nrArgs; ++a)
+        {
+            argDef *ad = &od->pysig.args[a];
+
+            if (!isOutArg(ad))
+                continue;
+
+            need_comma = apiArgument(ad, TRUE, need_comma, sec, fp);
+        }
+
+        if ((is_res && nr_out > 0) || nr_out > 1)
+            fprintf(fp, ")");
+    }
 
     fprintf(fp, "\n");
 
@@ -252,8 +288,7 @@ static int apiOverload(sipSpec *pt, moduleDef *mod, classDef *scope,
 /*
  * Generate the API for an argument.
  */
-static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
-        int sec, int names, int defaults, int in_str, FILE *fp)
+static int apiArgument(argDef *ad, int out, int need_comma, int sec, FILE *fp)
 {
     const char *tname;
     classDef *tscope;
@@ -264,7 +299,7 @@ static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
     if (sec && (ad->atype == slotcon_type || ad->atype == slotdis_type))
         return need_comma;
 
-    if ((tname = pyType(pt, ad, sec, &tscope)) == NULL)
+    if ((tname = pyType(ad, sec, &tscope)) == NULL)
         return need_comma;
 
     if (need_comma)
@@ -272,17 +307,19 @@ static int apiArgument(sipSpec *pt, argDef *ad, int out, int need_comma,
 
     prScopedPythonName(fp, tscope, tname);
 
-    /*
-     * Handle the default value is required, but ignore it if it is an output
-     * only argument.
-     */
-    if (defaults && ad->defval && !out)
-    {
-        if (names && ad->name != NULL)
-            fprintf(fp, " %s", ad->name->text);
+    if (ad->name != NULL)
+        fprintf(fp, " %s", ad->name);
 
+    /*
+     * Handle the default value, but ignore it if it is an output only
+     * argument.
+     */
+    if (ad->defval && !out)
+    {
         fprintf(fp, "=");
-        prDefaultValue(ad, in_str, fp);
+        prcode(fp, "%M");
+        exportDefaultValue(ad, fp);
+        prcode(fp, "%M");
     }
 
     return TRUE;
@@ -327,7 +364,7 @@ void generateXML(sipSpec *pt, moduleDef *mod, const char *xmlFile)
     xmlVars(pt, mod, NULL, 1, fp);
 
     for (md = mod->othfuncs; md != NULL; md = md->next)
-        xmlFunction(pt, NULL, md, mod->overs, 1, fp);
+        xmlFunction(NULL, md, mod->overs, 1, fp);
 
     fprintf(fp, "</Module>\n");
 
@@ -347,72 +384,60 @@ static void xmlClass(sipSpec *pt, moduleDef *mod, classDef *cd, FILE *fp)
     if (isOpaque(cd))
     {
         xmlIndent(indent, fp);
-        fprintf(fp, "<OpaqueClass name=\"");
-        prScopedPythonName(fp, cd->ecd, cd->pyname->text);
+        fprintf(fp, "<OpaqueClass name=\"", cd->pyname);
+        prScopedPythonName(fp, cd->ecd, cd->pyname);
         fprintf(fp, "\"/>\n");
 
         return;
     }
 
-    if (!isHiddenNamespace(cd))
+    xmlIndent(indent++, fp);
+    fprintf(fp, "<Class name=\"", cd->pyname);
+    prScopedPythonName(fp, cd->ecd, cd->pyname);
+    fprintf(fp, "\"");
+
+    if (cd->convtocode != NULL)
+        fprintf(fp, " convert=\"1\"");
+
+    if (cd->real != NULL)
+        fprintf(fp, " extends=\"%s\"", cd->real->iff->module->name);
+
+    if (cd->supers != NULL)
     {
-        xmlIndent(indent++, fp);
-        fprintf(fp, "<Class name=\"");
-        prScopedPythonName(fp, cd->ecd, cd->pyname->text);
-        fprintf(fp, "\"");
+        classList *cl;
 
-        if (cd->picklecode != NULL)
-            fprintf(fp, " pickle=\"1\"");
+        fprintf(fp, " inherits=\"");
 
-        if (cd->convtocode != NULL)
-            fprintf(fp, " convert=\"1\"");
-
-        if (cd->convfromcode != NULL)
-            fprintf(fp, " convertfrom=\"1\"");
-
-        if (cd->real != NULL)
-            fprintf(fp, " extends=\"%s\"", cd->real->iff->module->name);
-
-        if (cd->supers != NULL)
+        for (cl = cd->supers; cl != NULL; cl = cl->next)
         {
-            classList *cl;
+            if (cl != cd->supers)
+                fprintf(fp, " ");
 
-            fprintf(fp, " inherits=\"");
-
-            for (cl = cd->supers; cl != NULL; cl = cl->next)
-            {
-                if (cl != cd->supers)
-                    fprintf(fp, " ");
-
-                prScopedPythonName(fp, cl->cd->ecd, cl->cd->pyname->text);
-            }
-
-            fprintf(fp, "\"");
+            prScopedPythonName(fp, cl->cd->ecd, cl->cd->pyname);
         }
 
-        fprintf(fp, ">\n");
+        fprintf(fp, "\"");
     }
 
+    fprintf(fp, ">\n");
+
+    xmlEnums(pt, mod, cd, indent, fp);
+    xmlVars(pt, mod, cd, indent, fp);
+
     for (ct = cd->ctors; ct != NULL; ct = ct->next)
     {
         if (isPrivateCtor(ct))
             continue;
 
-        if (xmlCtor(pt, cd, ct, FALSE, indent, fp))
-            xmlCtor(pt, cd, ct, TRUE, indent, fp);
+        if (xmlCtor(cd, ct, FALSE, indent, fp))
+            xmlCtor(cd, ct, TRUE, indent, fp);
     }
 
-    xmlEnums(pt, mod, cd, indent, fp);
-    xmlVars(pt, mod, cd, indent, fp);
-
     for (md = cd->members; md != NULL; md = md->next)
-        xmlFunction(pt, cd, md, cd->overs, indent, fp);
+        xmlFunction(cd, md, cd->overs, indent, fp);
 
-    if (!isHiddenNamespace(cd))
-    {
-        xmlIndent(--indent, fp);
-        fprintf(fp, "</Class>\n");
-    }
+    xmlIndent(--indent, fp);
+    fprintf(fp, "</Class>\n");
 }
 
 
@@ -495,7 +520,7 @@ static void xmlVars(sipSpec *pt, moduleDef *mod, classDef *scope, int indent,
         if (isStaticVar(vd))
             fprintf(fp, " static=\"1\"");
 
-        xmlType(pt, &vd->type, FALSE, fp);
+        xmlType(&vd->type, FALSE, fp);
         fprintf(fp, "/>\n");
     }
 }
@@ -504,8 +529,7 @@ static void xmlVars(sipSpec *pt, moduleDef *mod, classDef *scope, int indent,
 /*
  * Generate the XML for a ctor.
  */
-static int xmlCtor(sipSpec *pt, classDef *scope, ctorDef *ct, int sec,
-        int indent, FILE *fp)
+static int xmlCtor(classDef *scope, ctorDef *ct, int sec, int indent, FILE *fp)
 {
     int a, need_sec;
 
@@ -529,7 +553,7 @@ static int xmlCtor(sipSpec *pt, classDef *scope, ctorDef *ct, int sec,
     {
         argDef *ad = &ct->pysig.args[a];
 
-        xmlArgument(pt, ad, dirAttribute(ad), FALSE, sec, indent, fp);
+        xmlArgument(ad, dirAttribute(ad), FALSE, sec, indent, fp);
 
         if (ad->atype == rxcon_type || ad->atype == rxdis_type)
             need_sec = TRUE;
@@ -545,11 +569,10 @@ static int xmlCtor(sipSpec *pt, classDef *scope, ctorDef *ct, int sec,
 /*
  * Generate the XML for a function.
  */
-static void xmlFunction(sipSpec *pt, classDef *scope, memberDef *md,
-        overDef *oloads, int indent, FILE *fp)
+static void xmlFunction(classDef *scope, memberDef *md, overDef *oloads,
+        int indent, FILE *fp)
 {
     overDef *od;
-    const char *default_str = "default=\"1\" ";
 
     for (od = oloads; od != NULL; od = od->next)
     {
@@ -565,13 +588,11 @@ static void xmlFunction(sipSpec *pt, classDef *scope, memberDef *md,
         if (isSignal(od))
         {
             xmlIndent(indent, fp);
-            fprintf(fp, "<Signal %sname=\"", default_str);
+            fprintf(fp, "<Signal name=\"");
             prScopedPythonName(fp, scope, md->pyname->text);
             fprintf(fp, "\" sig=\"");
             xmlCppSignature(fp, od);
-            fprintf(fp, "\"/>\n");
-
-            default_str = "";
+            fprintf(fp, "\"/>\n", md->pyname->text);
 
             continue;
         }
@@ -585,8 +606,8 @@ static void xmlFunction(sipSpec *pt, classDef *scope, memberDef *md,
             isstat = FALSE;
         }
 
-        if (xmlOverload(pt, scope, md, od, xtnds, isstat, FALSE, indent, fp))
-            xmlOverload(pt, scope, md, od, xtnds, isstat, TRUE, indent, fp);
+        if (xmlOverload(scope, md, od, xtnds, isstat, FALSE, indent, fp))
+            xmlOverload(scope, md, od, xtnds, isstat, TRUE, indent, fp);
     }
 }
 
@@ -594,8 +615,8 @@ static void xmlFunction(sipSpec *pt, classDef *scope, memberDef *md,
 /*
  * Generate the XML for an overload.
  */
-static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
-        overDef *od, classDef *xtnds, int stat, int sec, int indent, FILE *fp)
+static int xmlOverload(classDef *scope, memberDef *md, overDef *od,
+        classDef *xtnds, int stat, int sec, int indent, FILE *fp)
 {
     int a, need_sec, no_res;
 
@@ -617,15 +638,10 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
         fprintf(fp, "\"");
     }
 
-    if (isVirtual(od))
-    {
-        fprintf(fp, " virtual=\"1\"");
-    }
-
     if (xtnds != NULL)
     {
         fprintf(fp, " extends=\"");
-        prScopedPythonName(fp, xtnds->ecd, xtnds->pyname->text);
+        prScopedPythonName(fp, xtnds->ecd, xtnds->pyname);
         fprintf(fp, "\"");
     }
 
@@ -641,8 +657,7 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
     fprintf(fp, ">\n");
 
     if (!no_res)
-        xmlArgument(pt, &od->pysig.result, "out", isResultTransferredBack(od),
-                FALSE, indent, fp);
+        xmlArgument(&od->pysig.result, "out", isResultTransferredBack(od), FALSE, indent, fp);
 
     need_sec = FALSE;
 
@@ -654,7 +669,7 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
         if (isNumberSlot(md) && a == 0 && od->pysig.nrArgs == 2)
             continue;
 
-        xmlArgument(pt, ad, dirAttribute(ad), FALSE, sec, indent, fp);
+        xmlArgument(ad, dirAttribute(ad), FALSE, sec, indent, fp);
 
         if (ad->atype == rxcon_type || ad->atype == rxdis_type)
             need_sec = TRUE;
@@ -673,7 +688,7 @@ static int xmlOverload(sipSpec *pt, classDef *scope, memberDef *md,
 static void xmlCppSignature(FILE *fp, overDef *od)
 {
     prcode(fp, "%M");
-    prOverloadDecl(fp, NULL, od, TRUE);
+    prOverloadDecl(fp, od, TRUE);
     prcode(fp, "%M");
 }
 
@@ -698,8 +713,8 @@ static const char *dirAttribute(argDef *ad)
 /*
  * Generate the XML for an argument.
  */
-static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
-        int sec, int indent, FILE *fp)
+static void xmlArgument(argDef *ad, const char *dir, int res_xfer, int sec,
+        int indent, FILE *fp)
 {
     if (isArraySize(ad))
         return;
@@ -709,7 +724,7 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
 
     xmlIndent(indent, fp);
     fprintf(fp, "<Argument");
-    xmlType(pt, ad, sec, fp);
+    xmlType(ad, sec, fp);
 
     if (dir != NULL)
         fprintf(fp, " dir=\"%s\"", dir);
@@ -717,9 +732,6 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
     if (isAllowNone(ad))
         fprintf(fp, " allownone=\"1\"");
 
-    if (isDisallowNone(ad))
-        fprintf(fp, " disallownone=\"1\"");
-
     if (isTransferred(ad))
         fprintf(fp, " transfer=\"to\"");
     else if (isThisTransferred(ad))
@@ -733,9 +745,9 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
      */
     if (ad->defval && (dir == NULL || strcmp(dir, "out") != 0))
     {
-        prcode(fp, " default=\"");
-        prDefaultValue(ad, FALSE, fp);
-        prcode(fp, "\"");
+        prcode(fp, " default=\"%M");
+        exportDefaultValue(ad, fp);
+        prcode(fp, "%M\"");
     }
 
     fprintf(fp, "/>\n");
@@ -745,7 +757,7 @@ static void xmlArgument(sipSpec *pt, argDef *ad, const char *dir, int res_xfer,
 /*
  * Generate the XML for a type.
  */
-static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp)
+static void xmlType(argDef *ad, int sec, FILE *fp)
 {
     const char *type_type = NULL, *type_name;
     classDef *type_scope;
@@ -794,15 +806,12 @@ static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp)
         break;
 
     case mapped_type:
+        prcode(fp, "%M%B%M", &ad->u.mtd->type);
         type_type = "mappedtype";
         break;
-
-    /* Suppress a compiler warning. */
-    default:
-        ;
     }
 
-    if ((type_name = pyType(pt, ad, sec, &type_scope)) != NULL)
+    if ((type_name = pyType(ad, sec, &type_scope)) != NULL)
         prScopedPythonName(fp, type_scope, type_name);
 
     fprintf(fp, "\"");
@@ -811,7 +820,7 @@ static void xmlType(sipSpec *pt, argDef *ad, int sec, FILE *fp)
         fprintf(fp, " typetype=\"%s\"", type_type);
 
     if (ad->name != NULL)
-        fprintf(fp, " name=\"%s\"", ad->name->text);
+        fprintf(fp, " name=\"%s\"", ad->name);
 }
 
 
@@ -825,111 +834,45 @@ static void xmlIndent(int indent, FILE *fp)
 }
 
 
+/*
+ * Export the default value of an argument.
+ */
+static void exportDefaultValue(argDef *ad, FILE *fp)
+{
+    /* Translate some special cases. */
+    if (ad->defval->next == NULL && ad->defval->vtype == numeric_value)
+    {
+        if (ad->nrderefs > 0 && ad->defval->u.vnum == 0)
+        {
+            prcode(fp, "None");
+            return;
+        }
+
+        if (ad->atype == bool_type || ad->atype == cbool_type)
+        {
+            prcode(fp, ad->defval->u.vnum ? "True" : "False");
+            return;
+        }
+    }
+
+    generateExpression(ad->defval, fp);
+}
+
+
 /*
  * Get the Python representation of a type.
  */
-static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
+static const char *pyType(argDef *ad, int sec, classDef **scope)
 {
     const char *type_name;
 
     *scope = NULL;
 
-    /* Use any explicit documented type. */
-    if (ad->doctype != NULL)
-        return ad->doctype;
-
-    /* For classes and mapped types we need the default implementation. */
-    if (ad->atype == class_type || ad->atype == mapped_type)
-    {
-        classDef *def_cd = NULL;
-        mappedTypeDef *def_mtd = NULL;
-        ifaceFileDef *iff;
-
-        if (ad->atype == class_type)
-        {
-            iff = ad->u.cd->iff;
-
-            if (iff->api_range == NULL)
-            {
-                /* There is only one implementation. */
-                def_cd = ad->u.cd;
-                iff = NULL;
-            }
-        }
-        else
-        {
-            iff = ad->u.mtd->iff;
-
-            if (iff->api_range == NULL)
-            {
-                /* There is only one implementation. */
-                def_mtd = ad->u.mtd;
-                iff = NULL;
-            }
-        }
-
-        if (iff != NULL)
-        {
-            int def_api;
-
-            /* Find the default implementation. */
-            def_api = findAPI(pt, iff->api_range->api_name->text)->from;
-
-            for (iff = iff->first_alt; iff != NULL; iff = iff->next_alt)
-            {
-                apiVersionRangeDef *avd = iff->api_range;
-
-                if (avd->from > 0 && avd->from > def_api)
-                    continue;
-
-                if (avd->to > 0 && avd->to <= def_api)
-                    continue;
-
-                /* It's within range. */
-                break;
-            }
-
-            /* Find the corresponding class or mapped type. */
-            for (def_cd = pt->classes; def_cd != NULL; def_cd = def_cd->next)
-                if (def_cd->iff == iff)
-                    break;
-
-            if (def_cd == NULL)
-                for (def_mtd = pt->mappedtypes; def_mtd != NULL; def_mtd = def_mtd->next)
-                    if (def_mtd->iff == iff)
-                        break;
-        }
-
-        /* Now handle the correct implementation. */
-        if (def_cd != NULL)
-        {
-            *scope = def_cd->ecd;
-            type_name = def_cd->pyname->text;
-        }
-        else
-        {
-            /*
-             * Give a hint that /DocType/ should be used, or there is no
-             * default implementation.
-             */
-            type_name = "unknown-type";
-
-            if (def_mtd != NULL)
-            {
-                if (def_mtd->doctype != NULL)
-                    type_name = def_mtd->doctype;
-                else if (def_mtd->pyname != NULL)
-                    type_name = def_mtd->pyname->text;
-            }
-        }
-
-        return type_name;
-    }
-
     switch (ad->atype)
     {
-    case capsule_type:
-        type_name = scopedNameTail(ad->u.cap);
+    case class_type:
+        type_name = ad->u.cd->pyname;
+        *scope = ad->u.cd->ecd;
         break;
 
     case struct_type:
@@ -969,35 +912,34 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
         break;
 
     case ustring_type:
-        /* Correct for Python v3. */
-        type_name = "bytes";
-        break;
-
     case string_type:
     case sstring_type:
     case wstring_type:
-    case ascii_string_type:
-    case latin1_string_type:
-    case utf8_string_type:
-        type_name = isArray(ad) ? "bytes" : "str";
+        if (ad->nrderefs > 0)
+            type_name = "str";
+        else
+            type_name = "char";
+
         break;
 
-    case byte_type:
-    case sbyte_type:
-    case ubyte_type:
-    case ushort_type:
-    case uint_type:
-    case long_type:
-    case longlong_type:
-    case ulong_type:
-    case ulonglong_type:
     case short_type:
     case int_type:
     case cint_type:
-    case ssize_type:
         type_name = "int";
         break;
 
+    case ushort_type:
+    case uint_type:
+    case ulong_type:
+    case ulonglong_type:
+        type_name = "unsigned long";
+        break;
+
+    case long_type:
+    case longlong_type:
+        type_name = "long";
+        break;
+
     case float_type:
     case cfloat_type:
     case double_type:
@@ -1038,16 +980,12 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
         type_name = "type";
         break;
 
-    case pybuffer_type:
-        type_name = "buffer";
-        break;
-
     case ellipsis_type:
         type_name = "...";
         break;
 
-    case slotcon_type:
     case anyslot_type:
+        /* Need to check if this is enough. */
         type_name = "SLOT()";
         break;
 
@@ -1057,85 +995,3 @@ static const char *pyType(sipSpec *pt, argDef *ad, int sec, classDef **scope)
 
     return type_name;
 }
-
-
-/*
- * Generate a Python signature.
- */
-static int exportPythonSignature(sipSpec *pt, FILE *fp, signatureDef *sd,
-        int sec, int names, int defaults, int in_str, int is_signal)
-{
-    int need_sec = FALSE, need_comma = FALSE, is_res, nr_out, a;
-
-    if (is_signal)
-    {
-        if (sd->nrArgs != 0)
-            fprintf(fp, "[");
-    }
-    else
-    {
-        fprintf(fp, "(");
-    }
-
-    nr_out = 0;
-
-    for (a = 0; a < sd->nrArgs; ++a)
-    {
-        argDef *ad = &sd->args[a];
-
-        if (isOutArg(ad))
-            ++nr_out;
-
-        if (!isInArg(ad))
-            continue;
-
-        need_comma = apiArgument(pt, ad, FALSE, need_comma, sec, names,
-                defaults, in_str, fp);
-
-        if (ad->atype == rxcon_type || ad->atype == rxdis_type)
-            need_sec = TRUE;
-    }
-
-    if (is_signal)
-    {
-        if (sd->nrArgs != 0)
-            fprintf(fp, "]");
-    }
-    else
-    {
-        fprintf(fp, ")");
-    }
-
-
-    is_res = !((sd->result.atype == void_type && sd->result.nrderefs == 0) ||
-            (sd->result.doctype != NULL && sd->result.doctype[0] == '\0'));
-
-    if (is_res || nr_out > 0)
-    {
-        fprintf(fp, " -> ");
-
-        if ((is_res && nr_out > 0) || nr_out > 1)
-            fprintf(fp, "(");
-
-        if (is_res)
-            need_comma = apiArgument(pt, &sd->result, TRUE, FALSE, sec, FALSE,
-                    FALSE, in_str, fp);
-        else
-            need_comma = FALSE;
-
-        for (a = 0; a < sd->nrArgs; ++a)
-        {
-            argDef *ad = &sd->args[a];
-
-            if (isOutArg(ad))
-                /* We don't want the name in the result tuple. */
-                need_comma = apiArgument(pt, ad, TRUE, need_comma, sec, FALSE,
-                        FALSE, in_str, fp);
-        }
-
-        if ((is_res && nr_out > 0) || nr_out > 1)
-            fprintf(fp, ")");
-    }
-
-    return need_sec;
-}
