@@ -3,7 +3,7 @@
  * corresponding wrapped Python object.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -101,7 +101,7 @@ static sipHashEntry *findHashEntry(sipObjectMap *om,void *key)
  * Return the wrapped Python object of a specific type for a C/C++ address or
  * NULL if it wasn't found.
  */
-sipWrapper *sipOMFindObject(sipObjectMap *om,void *key, sipWrapperType *type)
+sipWrapper *sipOMFindObject(sipObjectMap *om, void *key, sipWrapperType *type)
 {
     sipHashEntry *he = findHashEntry(om, key);
     sipWrapper *w;
@@ -110,12 +110,10 @@ sipWrapper *sipOMFindObject(sipObjectMap *om,void *key, sipWrapperType *type)
     for (w = he->first; w != NULL; w = w->next)
     {
         /*
-         * If this wrapped object is of the given type, or a sub-type
-         * of it, or vice versa, then we assume it is the same C++
-         * object.
+         * If this wrapped object is of the given type, or a sub-type of it,
+         * then we assume it is the same C++ object.
          */
-        if (PyObject_TypeCheck(w, (PyTypeObject *)type) ||
-                PyType_IsSubtype((PyTypeObject *)type, w->ob_type))
+        if (PyObject_TypeCheck(w, (PyTypeObject *)type))
             return w;
     }
 
@@ -131,36 +129,42 @@ void sipOMAddObject(sipObjectMap *om,sipWrapper *val)
     sipHashEntry *he = findHashEntry(om,val -> u.cppPtr);
 
     /*
-     * If the bucket is in use then we appear to have several objects at
-     * the same address.
+     * If the bucket is in use then we appear to have several objects at the
+     * same address.
      */
     if (he -> first != NULL)
     {
         /*
-         * This can happen for three reasons.  A variable of one class
-         * can be declared at the start of another class.  Therefore
-         * there are two objects, of different classes, with the same
-         * address.  The second reason is that the old C/C++ object has
-         * been deleted by C/C++ but we didn't get to find out for some
-         * reason, and a new C/C++ instance has been created at the
-         * same address.  The third reason is if we are in the process
-         * of deleting a Python object but the C++ object gets wrapped
-         * again because the C++ dtor called a method that has been
-         * re-implemented in Python.  The absence of the SIP_SHARE_MAP
-         * flag tells us that a new C++ instance has just been created
-         * and so we know the second reason is the correct one so we
-         * mark the old pointers as invalid and reuse the entry.
-         * Otherwise we just add this one to the existing list of
-         * objects at this address.
+         * This can happen for three reasons.  A variable of one class can be
+         * declared at the start of another class.  Therefore there are two
+         * objects, of different classes, with the same address.  The second
+         * reason is that the old C/C++ object has been deleted by C/C++ but we
+         * didn't get to find out for some reason, and a new C/C++ instance has
+         * been created at the same address.  The third reason is if we are in
+         * the process of deleting a Python object but the C++ object gets
+         * wrapped again because the C++ dtor called a method that has been
+         * re-implemented in Python.  The absence of the SIP_SHARE_MAP flag
+         * tells us that a new C++ instance has just been created and so we
+         * know the second reason is the correct one so we mark the old
+         * pointers as invalid and reuse the entry.  Otherwise we just add this
+         * one to the existing list of objects at this address.
          */
         if (!(val -> flags & SIP_SHARE_MAP))
         {
-            sipWrapper *w;
+            sipWrapper *w = he->first;
 
-            for (w = he -> first; w != NULL; w = w -> next)
-                w -> u.cppPtr = NULL;
+            he->first = NULL;
 
-            he -> first = NULL;
+            while (w != NULL)
+            {
+                sipWrapper *next = w->next;
+
+                /* We are removing it from the map here. */
+                sipSetNotInMap(w);
+                sip_api_common_dtor(w);
+
+                w = next;
+            }
         }
 
         val -> next = he -> first;
