@@ -3,7 +3,7 @@
  * supplied Qt support.
  *
  * Copyright (c) 2007
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -454,6 +454,9 @@ static void *createUniversalSlot(sipWrapper *txSelf, const char *sig,
         return 0;
     }
 
+    if (txSelf)
+        sipSetPossibleProxy(txSelf);
+
     return us;
 }
 
@@ -584,7 +587,16 @@ int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs)
         return sip_api_emit_signal(slot -> pyobj,slot -> name,sigargs);
 
     /* Get the object to call, resolving any weak references. */
-    if (slot -> weakSlot == NULL)
+    if (slot->weakSlot == Py_True)
+    {
+        /*
+         * The slot is guaranteed to be Ok because it has an extra reference or
+         * is None.
+         */
+        sref = slot->pyobj;
+        Py_INCREF(sref);
+    }
+    else if (slot -> weakSlot == NULL)
         sref = NULL;
     else if ((sref = PyWeakref_GetObject(slot -> weakSlot)) == NULL)
         return -1;
@@ -649,15 +661,6 @@ int sip_api_emit_to_slot(sipSlot *slot, PyObject *sigargs)
         /* Make sure we garbage collect the new method. */
         newmeth = sfunc;
     }
-    else if (slot->pyobj == Py_None)
-    {
-        /*
-         * This was a lambda function that has been freed by the cyclic garbage
-         * collector so ignore it.
-         */
-        Py_XDECREF(sref);
-        return 0;
-    }
     else
     {
         sfunc = slot -> pyobj;
@@ -858,30 +861,40 @@ static int addSlotToPySigList(sipWrapper *txSelf,const char *sig,
 /*
  * Compare two slots to see if they are the same.
  */
-static int isSameSlot(sipSlot *slot1,PyObject *rxobj2,const char *slot2)
+static int isSameSlot(sipSlot *slot1, PyObject *rxobj2, const char *slot2)
 {
     /* See if they are signals or Qt slots, ie. they have a name. */
-    if (slot1 -> name != NULL)
-        return (slot2 != NULL &&
-            sipQtSupport->qt_same_name(slot1 -> name,slot2) &&
-            slot1 -> pyobj == rxobj2);
-
-    /* Both must be Python slots. */
     if (slot2 != NULL)
-        return 0;
+    {
+        if (slot1->name == NULL || slot1->name[0] == '\0')
+            return 0;
 
-    /* See if they are Python methods. */
-    if (slot1 -> pyobj == NULL)
-        return (PyMethod_Check(rxobj2) &&
-            slot1 -> meth.mfunc == PyMethod_GET_FUNCTION(rxobj2) &&
-            slot1 -> meth.mself == PyMethod_GET_SELF(rxobj2) &&
-            slot1 -> meth.mclass == PyMethod_GET_CLASS(rxobj2));
+        return (sipQtSupport->qt_same_name(slot1->name, slot2) && slot1->pyobj == rxobj2);
+    }
 
+    /* See if they are pure Python methods. */
     if (PyMethod_Check(rxobj2))
-        return 0;
+    {
+        if (slot1->pyobj != NULL)
+            return 0;
+
+        return (slot1->meth.mfunc == PyMethod_GET_FUNCTION(rxobj2) &&
+                slot1->meth.mself == PyMethod_GET_SELF(rxobj2) &&
+                slot1->meth.mclass == PyMethod_GET_CLASS(rxobj2));
+    }
+
+    /* See if they are wrapped C++ methods. */
+    if (PyCFunction_Check(rxobj2))
+    {
+        if (slot1->name == NULL || slot1->name[0] != '\0')
+            return 0;
+
+        return (slot1->pyobj == PyCFunction_GET_SELF(rxobj2) &&
+                strcmp(&slot1->name[1], ((PyCFunctionObject *)rxobj2)->m_ml->ml_name) == 0);
+    }
 
     /* The objects must be the same. */
-    return (slot1 -> pyobj == rxobj2);
+    return (slot1->pyobj == rxobj2);
 }
 
 
@@ -1068,13 +1081,8 @@ static void freeSlot(sipSlot *slot)
 {
     if (slot->name != NULL)
         sip_api_free(slot->name);
-    else
-    {
-        PyObject *lam = slot->pyobj;
-
-        if (lam != NULL && (lam == Py_None || sipLambdaSlot(lam)))
-            Py_DECREF(lam);
-    }
+    else if (slot->weakSlot == Py_True)
+        Py_DECREF(slot->pyobj);
 
     /* Remove any weak reference. */
     Py_XDECREF(slot->weakSlot);
@@ -1177,20 +1185,14 @@ static int saveSlot(sipSlot *sp, PyObject *rxObj, const char *slot)
             else
             {
                 /*
-                 * A bit of a hack to allow lamba functions to be used as
-                 * slots.
+                 * Give the slot an extra reference to keep it alive and
+                 * remember we have done so by treating weakSlot specially.
                  */
-                if (sipLambdaSlot(rxObj))
-                    Py_INCREF(rxObj);
+                Py_INCREF(rxObj);
+                sp->pyobj = rxObj;
 
-                /*
-                 * It's unlikely that we will succeed in getting a weak
-                 * reference to the slot, but there is no harm in trying (and
-                 * future versions of Python may support references to more
-                 * object types).
-                 */
-                sp -> pyobj = rxObj;
-                sp -> weakSlot = getWeakRef(rxObj);
+                Py_INCREF(Py_True);
+                sp->weakSlot = Py_True;
             }
         }
     }
@@ -1240,15 +1242,3 @@ static PyObject *getWeakRef(PyObject *obj)
 
     return wr;
 }
-
-
-/*
- * See if an object is a lambda function.
- */
-int sipLambdaSlot(PyObject *slotObj)
-{
-    if (!PyFunction_Check(slotObj))
-        return FALSE;
-
-    return (strcmp(PyString_AsString(((PyFunctionObject *)slotObj)->func_name), "<lambda>") == 0);
-}
