@@ -1,8 +1,8 @@
 /*
  * The SIP module interface.
  *
- * Copyright (c) 2006
- * 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+ * Copyright (c) 2007
+ * 	Phil Thompson <phil@river-bank.demon.co.uk>
  * 
  * This file is part of SIP.
  * 
@@ -18,6 +18,14 @@
 #define _SIP_H
 
 
+/*
+ * This gets round a problem with Qt's moc and Python v2.3.  Strictly speaking
+ * it's a Qt problem but later versions of Python include a fix for it so we
+ * might as well too.
+ */
+#undef slots
+
+
 #include <Python.h>
 
 /*
@@ -43,8 +51,8 @@ extern "C" {
 /*
  * Define the SIP version number.
  */
-#define SIP_VERSION         0x040500
-#define SIP_VERSION_STR     "4.5"
+#define SIP_VERSION         0x040701
+#define SIP_VERSION_STR     "4.7.1"
 
 
 /*
@@ -57,6 +65,17 @@ extern "C" {
  *
  * History:
  *
+ * 3.5  Added the td_pickle field to the sipTypeDef structure.
+ *      Added sip_api_transfer_break().
+ *
+ * 3.4  Added qt_find_connection() to the Qt support API.
+ *      Added sip_api_string_as_char(), sip_api_unicode_as_wchar(),
+ *      sip_api_unicode_as_wstring(), sip_api_find_class(),
+ *      sip_api_find_named_enum() and sip_api_parse_signature().
+ *      Added the 'A', 'w' and 'x' format characters to sip_api_parse_args(),
+ *      sip_api_parse_result(), sip_api_build_result() and
+ *      sip_api_call_method().
+ *
  * 3.3  Added sip_api_register_int_types().
  *
  * 3.2  Added sip_api_export_symbol() and sip_api_import_symbol().
@@ -93,7 +112,7 @@ extern "C" {
  * 0.0  Original version.
  */
 #define SIP_API_MAJOR_NR    3
-#define SIP_API_MINOR_NR    2
+#define SIP_API_MINOR_NR    5
 
 
 /* Some compatibility stuff to help with handwritten code for SIP v3. */
@@ -102,11 +121,11 @@ extern "C" {
 #endif
 
 
-/* Some internal compatibility stuff. */
+/* Some Python compatibility stuff. */
 #if PY_VERSION_HEX >= 0x02050000
-#define _SIP_SSIZE_T        Py_ssize_t
+#define SIP_SSIZE_T         Py_ssize_t
 #else
-#define _SIP_SSIZE_T        int
+#define SIP_SSIZE_T         int
 #endif
 
 
@@ -152,6 +171,9 @@ typedef struct _sipWrapperType {
 
     /* The list of init extenders. */
     struct _sipInitExtenderDef *iextend;
+
+    /* The optional PyQt defined Q_OBJECT information. */
+    void *qt_qobject;
 } sipWrapperType;
 
 
@@ -204,8 +226,8 @@ typedef struct _sipWrapper {
 typedef void *(*sipInitFunc)(sipWrapper *, PyObject *, sipWrapper **, int *);
 typedef int (*sipTraverseFunc)(void *, visitproc, void *);
 typedef int (*sipClearFunc)(void *);
-typedef int (*sipBufferFunc)(PyObject *, void *, int, void **);
-typedef int (*sipSegCountFunc)(PyObject *, void *, int *);
+typedef SIP_SSIZE_T (*sipBufferFunc)(PyObject *, void *, SIP_SSIZE_T, void **);
+typedef SIP_SSIZE_T (*sipSegCountFunc)(PyObject *, void *, SIP_SSIZE_T *);
 typedef void (*sipDeallocFunc)(sipWrapper *);
 typedef void *(*sipCastFunc)(void *, sipWrapperType *);
 typedef sipWrapperType *(*sipSubClassConvertFunc)(void **);
@@ -215,6 +237,7 @@ typedef PyObject *(*sipConvertFromFunc)(void *, PyObject *);
 typedef int (*sipVirtHandlerFunc)(void *, PyObject *, ...);
 typedef int (*sipEmitFunc)(sipWrapper *, PyObject *);
 typedef void (*sipReleaseFunc)(void *, int);
+typedef PyObject *(*sipPickleFunc)(void *);
 
 
 /*
@@ -446,7 +469,9 @@ typedef enum {
     qvariantp_sat,
     pyobject_sat,
     schar_sat,
-    sstring_sat
+    sstring_sat,
+    wchar_sat,
+    wstring_sat
 } sipSigArgType;
 
 
@@ -552,6 +577,9 @@ typedef struct _sipTypeDef {
 
     /* The next namespace extender. */
     struct _sipTypeDef *td_nsextender;
+
+    /* The pickle function. */
+    sipPickleFunc td_pickle;
 } sipTypeDef;
 
 
@@ -942,21 +970,21 @@ typedef struct _sipSlot {
     /* Python slot method, pyobj is NULL. */
     sipPyMethod meth;
 
-    /* A weak reference to the slot. */
+    /* A weak reference to the slot, Py_True if pyobj has an extra reference. */
     PyObject *weakSlot;
 } sipSlot;
 
 
 /*
- * A receiver of a Python signal.
+ * An entry in a linked list of slots.
  */
-typedef struct _sipPySigRx {
+typedef struct _sipSlotList {
     /* The receiver. */
     sipSlot rx;
 
     /* Next in the list. */
-    struct _sipPySigRx *next;
-} sipPySigRx;
+    struct _sipSlotList *next;
+} sipSlotList;
 
 
 /*
@@ -967,7 +995,7 @@ typedef struct _sipPySig {
     char *name;
 
     /* The list of receivers. */
-    sipPySigRx *rxlist;
+    sipSlotList *rxlist;
 
     /* Next in the list. */
     struct _sipPySig *next;
@@ -1055,16 +1083,15 @@ typedef struct _sipAPIDef {
      * The following are part of the public API.
      */
     void (*api_bad_catcher_result)(PyObject *method);
-    void (*api_bad_length_for_slice)(_SIP_SSIZE_T seqlen,
-            _SIP_SSIZE_T slicelen);
+    void (*api_bad_length_for_slice)(SIP_SSIZE_T seqlen, SIP_SSIZE_T slicelen);
     PyObject *(*api_build_result)(int *isErr, const char *fmt, ...);
     PyObject *(*api_call_method)(int *isErr, PyObject *method, const char *fmt,
             ...);
     PyObject *(*api_class_name)(PyObject *self);
     PyObject *(*api_connect_rx)(PyObject *txObj, const char *sig,
             PyObject *rxObj, const char *slot, int type);
-    _SIP_SSIZE_T (*api_convert_from_sequence_index)(_SIP_SSIZE_T idx,
-            _SIP_SSIZE_T len);
+    SIP_SSIZE_T (*api_convert_from_sequence_index)(SIP_SSIZE_T idx,
+            SIP_SSIZE_T len);
     int (*api_can_convert_to_instance)(PyObject *pyObj, sipWrapperType *type,
             int flags);
     int (*api_can_convert_to_mapped_type)(PyObject *pyObj,
@@ -1152,16 +1179,16 @@ typedef struct _sipAPIDef {
     void (*api_raise_unknown_exception)(void);
     void (*api_raise_class_exception)(sipWrapperType *type, void *ptr);
     void (*api_raise_sub_class_exception)(sipWrapperType *type, void *ptr);
-    int (*api_add_class_instance)(PyObject *dict, char *name, void *cppPtr,
-            sipWrapperType *wt);
-    int (*api_add_enum_instance)(PyObject *dict, char *name, int value,
+    int (*api_add_class_instance)(PyObject *dict, const char *name,
+            void *cppPtr, sipWrapperType *wt);
+    int (*api_add_enum_instance)(PyObject *dict, const char *name, int value,
             PyTypeObject *type);
     void (*api_bad_operator_arg)(PyObject *self, PyObject *arg,
             sipPySlotType st);
     PyObject *(*api_pyslot_extend)(sipExportedModuleDef *mod, sipPySlotType st,
             sipWrapperType *type, PyObject *arg0, PyObject *arg1);
     void (*api_add_delayed_dtor)(sipWrapper *w);
-    int (*api_add_mapped_type_instance)(PyObject *dict, char *name,
+    int (*api_add_mapped_type_instance)(PyObject *dict, const char *name,
             void *cppPtr, const sipMappedType *mt);
 
     /*
@@ -1175,6 +1202,30 @@ typedef struct _sipAPIDef {
      * code.
      */
     int (*api_register_int_types)(PyObject *args);
+    sipSignature *(*api_parse_signature)(const char *sig);
+
+    /*
+     * The following are part of the public API.
+     */
+    sipWrapperType *(*api_find_class)(const char *type);
+    PyTypeObject *(*api_find_named_enum)(const char *type);
+
+    /*
+     * The following are not part of the public API.
+     */
+    char (*api_string_as_char)(PyObject *obj);
+#if defined(HAVE_WCHAR_H)
+    wchar_t (*api_unicode_as_wchar)(PyObject *obj);
+    wchar_t *(*api_unicode_as_wstring)(PyObject *obj);
+#else
+    int (*api_unicode_as_wchar)(PyObject *obj);
+    int *(*api_unicode_as_wstring)(PyObject *obj);
+#endif
+
+    /*
+     * The following are part of the public API.
+     */
+    void (*api_transfer_break)(PyObject *self);
 } sipAPIDef;
 
 
@@ -1203,6 +1254,7 @@ typedef struct _sipQtAPI {
     const void *(*qt_get_sender)();
     void (*qt_forget_sender)();
     int (*qt_same_name)(const char *, const char *);
+    sipSlotConnection *(*qt_find_connection)(void *, void **);
 } sipQtAPI;
 
 
@@ -1218,18 +1270,20 @@ typedef struct _sipQtAPI {
  * These are the state flags returned by %ConvertToTypeCode.  Note that these
  * share the same "namespace" as the flags below.
  */
-#define SIP_TEMPORARY       0x01    /* A temporary instance. */
-#define SIP_DERIVED_CLASS   0x02    /* The instance is derived. */
+#define SIP_TEMPORARY       0x0001  /* A temporary instance. */
+#define SIP_DERIVED_CLASS   0x0002  /* The instance is derived. */
 
 
 /*
  * Useful macros, not part of the public API.
  */
-#define SIP_PY_OWNED        0x04    /* Owned by Python. */
-#define SIP_INDIRECT        0x08    /* If there is a level of indirection. */
-#define SIP_ACCFUNC         0x10    /* If there is an access function. */
-#define SIP_NOT_IN_MAP      0x20    /* If Python object not in the map. */
-#define SIP_SHARE_MAP       0x40    /* If the map slot might be occupied. */
+#define SIP_PY_OWNED        0x0004  /* Owned by Python. */
+#define SIP_INDIRECT        0x0008  /* If there is a level of indirection. */
+#define SIP_ACCFUNC         0x0010  /* If there is an access function. */
+#define SIP_NOT_IN_MAP      0x0020  /* If Python object not in the map. */
+#define SIP_SHARE_MAP       0x0040  /* If the map slot might be occupied. */
+#define SIP_CPP_HAS_REF     0x0080  /* If C/C++ has a reference. */
+#define SIP_POSSIBLE_PROXY  0x0100  /* If there might be a proxy slot. */
 
 #define sipIsPyOwned(w)     ((w)->flags & SIP_PY_OWNED)
 #define sipSetPyOwned(w)    ((w)->flags |= SIP_PY_OWNED)
@@ -1238,13 +1292,22 @@ typedef struct _sipQtAPI {
 #define sipIsIndirect(w)    ((w)->flags & SIP_INDIRECT)
 #define sipIsAccessFunc(w)  ((w)->flags & SIP_ACCFUNC)
 #define sipNotInMap(w)      ((w)->flags & SIP_NOT_IN_MAP)
+#define sipSetNotInMap(w)   ((w)->flags |= SIP_NOT_IN_MAP)
+#define sipCppHasRef(w)     ((w)->flags & SIP_CPP_HAS_REF)
+#define sipSetCppHasRef(w)  ((w)->flags |= SIP_CPP_HAS_REF)
+#define sipResetCppHasRef(w)    ((w)->flags &= ~SIP_CPP_HAS_REF)
+#define sipPossibleProxy(w) ((w)->flags & SIP_POSSIBLE_PROXY)
+#define sipSetPossibleProxy(w)  ((w)->flags |= SIP_POSSIBLE_PROXY)
 
 
-#define SIP_TYPE_ABSTRACT   0x01    /* If the type is abstract. */
-#define SIP_TYPE_SCC        0x02    /* If the type is subject to sub-class convertors. */
+#define SIP_TYPE_ABSTRACT   0x0001  /* If the type is abstract. */
+#define SIP_TYPE_SCC        0x0002  /* If the type is subject to sub-class convertors. */
+#define SIP_TYPE_FLAGS_SHIFT    8   /* The user type flags shift. */
+#define SIP_TYPE_FLAGS_MASK 0x0f00  /* The user type flags mask. */
 
 #define sipTypeIsAbstract(wt)   ((wt)->type->td_flags & SIP_TYPE_ABSTRACT)
 #define sipTypeHasSCC(wt)   ((wt)->type->td_flags & SIP_TYPE_SCC)
+#define sipTypeFlags(wt)    (((wt)->type->td_flags & SIP_TYPE_FLAGS_MASK) >> SIP_TYPE_FLAGS_SHIFT)
 
 
 #ifdef __cplusplus
