@@ -2,8 +2,8 @@
 # extension modules created with SIP.  It provides information about file
 # locations, version numbers etc., and provides some classes and functions.
 #
-# Copyright (c) 2006
-# 	Riverbank Computing Limited <info@riverbankcomputing.co.uk>
+# Copyright (c) 2007
+# 	Phil Thompson <phil@river-bank.demon.co.uk>
 # 
 # This file is part of SIP.
 # 
@@ -111,6 +111,13 @@ class _UniqueList:
         if value not in self._list:
             self._list.append(value)
 
+    def lextend(self, value):
+        """A normal list extend ignoring the uniqueness.
+
+        value is the list of elements to append.
+        """
+        self._list.extend(value)
+
     def extend(self, value):
         """Append each element of a value to a list if it isn't already
         present.
@@ -181,7 +188,7 @@ class Makefile:
     """
     def __init__(self, configuration, console=0, qt=0, opengl=0, python=0,
                  threaded=0, warnings=1, debug=0, dir=None,
-                 makefile="Makefile", installs=None):
+                 makefile="Makefile", installs=None, universal=''):
         """Initialise an instance of the target.  All the macros are left
         unchanged allowing scripts to manipulate them at will.
 
@@ -203,6 +210,8 @@ class Makefile:
         part list, the first of which is the source and the second is the
         destination.  If the source is another list then it is a set of source
         files and the destination is a directory.
+        universal is the name of the SDK if the target is a MacOS/X universal
+        binary.
         """
         if qt:
             if not hasattr(configuration, "qt_version"):
@@ -220,6 +229,9 @@ class Makefile:
         else:
             self._threaded = threaded
 
+        if sys.platform != "darwin":
+            universal = ''
+
         self.config = configuration
         self.console = console
         self._qt = qt
@@ -230,6 +242,7 @@ class Makefile:
         self._dir = dir
         self._makefile = makefile
         self._installs = installs
+        self._universal = universal
 
         self._finalised = 0
 
@@ -292,6 +305,11 @@ class Makefile:
         to a Makefile.
         """
         # Extract the things we might need from the Windows Qt configuration.
+        # Note that we used to think that if Qt was built with exceptions, RTTI
+        # and STL support enabled then anything that linked against it also
+        # needed the same flags.  However, detecting this was broken for some
+        # time and nobody complained.  For the moment we'll leave the code in
+        # but it will never be used.
         if self._qt:
             wcfg = string.split(self.config.qt_winconfig)
             win_shared = ("shared" in wcfg)
@@ -329,6 +347,15 @@ class Makefile:
         libdir.extend(self.extra_lib_dirs)
         libdir.extend(self.optional_list("LIBDIR"))
 
+        # Handle MacOS/X universal binaries.
+        if self._universal:
+            unicflags = ('-arch ppc -arch i386 -isysroot %s' % self._universal).split()
+            unilflags = ('-arch ppc -arch i386 -Wl,-syslibroot,%s' % self._universal).split()
+
+            cflags.lextend(unicflags)
+            cxxflags.lextend(unicflags)
+            lflags.lextend(unilflags)
+
         # Don't use a unique list as libraries may need to be searched more
         # than once.  Also MacOS/X uses the form "-framework lib" so we don't
         # want to lose the multiple "-framework".
@@ -482,6 +509,8 @@ class Makefile:
                         defines.append("QT_NETWORK_LIB")
                     elif mod == "QtOpenGL":
                         defines.append("QT_OPENGL_LIB")
+                    elif mod == "QtScript":
+                        defines.append("QT_SCRIPT_LIB")
                     elif mod == "QtSql":
                         defines.append("QT_SQL_LIB")
                     elif mod == "QtTest":
@@ -512,10 +541,12 @@ class Makefile:
                     "QtGui":        ("QtCore", ),
                     "QtNetwork":    ("QtCore", ),
                     "QtOpenGL":     ("QtCore", "QtGui"),
+                    "QtScript":     ("QtCore", ),
                     "QtSql":        ("QtCore", ),
                     "QtSvg":        ("QtCore", "QtGui", "QtXml"),
                     "QtTest":       ("QtCore", "QtGui"),
                     "QtXml":        ("QtCore", ),
+                    "QtDesigner":   ("QtCore", "QtGui"),
                     "QAxContainer": ("QtCore", "QtGui")
                 }
 
@@ -598,7 +629,7 @@ class Makefile:
                         if mod == "QAxContainer":
                             incdir.append(os.path.join(qtincdir[0], "ActiveQt"))
                         elif self._is_framework(mod):
-                            if mod == "QtAssistant":
+                            if mod == "QtAssistant" and self.config.qt_version < 0x040202:
                                 mod = "QtAssistantClient"
 
                             incdir.append(os.path.join(libdir_qt[0], mod + ".framework", "Headers"))
@@ -655,7 +686,10 @@ class Makefile:
         mname is the name of the module.
         """
         if mname == "QtAssistant":
-            lib = "QtAssistantClient"
+            if self.config.qt_version >= 0x040202 and sys.platform == "darwin":
+                lib = mname
+            else:
+                lib = "QtAssistantClient"
         else:
             lib = mname
 
@@ -665,9 +699,10 @@ class Makefile:
             elif self.config.qt_version < 0x040200 or sys.platform == "darwin":
                 lib = lib + "_debug"
 
-        if sys.platform == "win32":
+        if sys.platform == "win32" and "shared" in string.split(self.config.qt_winconfig):
             if (mname in ("QtCore", "QtGui", "QtNetwork", "QtOpenGL",
-                          "QtSql", "QtSvg", "QtTest", "QtXml") or
+                          "QtScript", "QtSql", "QtSvg", "QtTest", "QtXml",
+                          "QtDesigner") or
                 (self.config.qt_version >= 0x040200 and mname == "QtAssistant")):
                 lib = lib + "4"
 
@@ -1213,7 +1248,7 @@ class ModuleMakefile(Makefile):
     def __init__(self, configuration, build_file, install_dir=None, static=0,
                  console=0, qt=0, opengl=0, threaded=0, warnings=1, debug=0,
                  dir=None, makefile="Makefile", installs=None, strip=1,
-                 export_all=0):
+                 export_all=0, universal=''):
         """Initialise an instance of a module Makefile.
 
         build_file is the file containing the target specific information.  If
@@ -1227,7 +1262,7 @@ class ModuleMakefile(Makefile):
         increases the size of the module and slows down module load times but
         may avoid problems with modules that use exceptions.  The default is 0.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs)
+        Makefile.__init__(self, configuration, console, qt, opengl, 1, threaded, warnings, debug, dir, makefile, installs, universal)
 
         self._build = self.parse_build_file(build_file)
         self._install_dir = install_dir
@@ -1397,7 +1432,7 @@ class ModuleMakefile(Makefile):
         if self.static:
             if self.generator in ("MSVC", "MSVC.NET", "BMAKE"):
                 mfile.write("LIB = %s\n" % self.required_string("LIB"))
-            if self.generator == "MINGW":
+            elif self.generator == "MINGW":
                 mfile.write("AR = %s\n" % self.required_string("LIB"))
                 self._ranlib = None
             else:
@@ -1530,14 +1565,15 @@ class ProgramMakefile(Makefile):
     """
     def __init__(self, configuration, build_file=None, install_dir=None,
                  console=0, qt=0, opengl=0, python=0, threaded=0, warnings=1,
-                 debug=0, dir=None, makefile="Makefile", installs=None):
+                 debug=0, dir=None, makefile="Makefile", installs=None,
+                 universal=''):
         """Initialise an instance of a program Makefile.
 
         build_file is the file containing the target specific information.  If
         it is a dictionary instead then its contents are validated.
         install_dir is the directory the target will be installed in.
         """
-        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs)
+        Makefile.__init__(self, configuration, console, qt, opengl, python, threaded, warnings, debug, dir, makefile, installs, universal)
 
         self._install_dir = install_dir
 
@@ -2018,7 +2054,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
     class qmake_build_file_reader:
         def __init__(self, filename):
             self.filename = filename
-            self.path = os.path.dirname(filename)
+            self.currentfile = None
             self.filestack = []
             self.pathstack = []
             self.cond_fname = None
@@ -2026,7 +2062,7 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
         def _openfile(self, filename):
             try:
-                self.currentfile = open(filename, 'r')
+                f = open(filename, 'r')
             except IOError, detail:
                 # If this file is conditional then don't raise an error.
                 if self.cond_fname == filename:
@@ -2034,8 +2070,11 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
 
                 error("Unable to open %s: %s" % (filename, detail))
 
-            self.filestack.append(self.currentfile)
-            self.pathstack.append(self.path)
+            if self.currentfile:
+                self.filestack.append(self.currentfile)
+                self.pathstack.append(self.path)
+
+            self.currentfile = f
             self.path = os.path.dirname(filename)
 
         def readline(self):
@@ -2191,7 +2230,10 @@ def parse_build_macros(filename, names, overrides=None, properties=None):
                 try:
                     value = raw[lhs]
                 except KeyError:
-                    error("%s: macro '%s' is not defined." % (filename, lhs))
+                    # We used to treat this as an error, but Qt v4.3.0 has at
+                    # least one case that refers to an undefined macro.  If
+                    # qmake handles it then this must be the correct behaviour.
+                    value = ""
 
             rhs = rhs[:mstart] + value + rhs[mend:]
             estart = string.find(rhs, "$$(")
@@ -2299,7 +2341,16 @@ def create_wrapper(script, wrapper, gui=0):
 
         wf.write("@\"%s\" \"%s\" %%1 %%2 %%3 %%4 %%5 %%6 %%7 %%8 %%9\n" % (exe, script))
     elif sys.platform == "darwin":
-        wf.write("exec %sw %s ${1+\"$@\"}\n" % (sys.executable, script))
+        # python, pythonw and sys.executable are all different images.  We
+        # would prefer to use the latter (because it includes the version
+        # number) but that would mean being unable to support the "gui"
+        # argument.
+        if gui:
+            exe = "pythonw"
+        else:
+            exe = "python"
+
+        wf.write("exec %s %s ${1+\"$@\"}\n" % (exe, script))
     else:
         wf.write("exec %s %s ${1+\"$@\"}\n" % (sys.executable, script))
 
